''' unit/regression tests for s_code_project_updater '''
import logging
import os
import unittest

import pytest

from astropy.time import Time

from shared.schema.src.schema import create_session, Project
from shared.support.src.support.logging import get_console_logger

from .test_projects import AlmaTestProject, ScodeTestProject, ScienceTestProject

_LOG = get_console_logger("scode_project_updater_tests", logging.DEBUG)

class UpdaterTestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls) -> None:
        os.environ['CAPO_PROFILE'] = 'local'

    # @pytest.mark.skip('test_create_scode_test_project')
    def test_create_scode_test_project(self):
        session = None
        try:
            session = create_session('SDM')
            project = ScodeTestProject()
            result  = None

            try:
                # see if fake project already in DB
                result = session.query(Project) \
                    .filter(Project.project_code ==
                            project.project_code) \
                    .first()
            except Exception as exc:
                pytest.fail(f'query failure: {exc}')

            if result is not None:
                # delete it so we can insert a virgin copy
                try:
                    session.delete(result)
                except Exception as exc:
                    pytest.fail(f'delete failure: {exc}')

            # insert fake project
            try:
                session.add(project)
            except Exception as exc:
                pytest.fail(f'insert failure: {exc}')

        except Exception as exc:
            pytest.fail(f'failed to create session: {exc}')
        finally:
            if session is not None:
                # 1. delete fake project, then
                session.close()


    # @pytest.mark.skip('test_create_science_project')
    def test_create_science_project(self):
        session = None
        try:
            session = create_session('SDM')
            project = ScienceTestProject()
            result  = None

            try:
                # see if fake project already in DB
                result = session.query(Project) \
                    .filter(Project.project_code ==
                            project.project_code) \
                    .first()
            except Exception as exc:
                pytest.fail(f'query failure: {exc}')

            if result is not None:
                # delete it so we can insert a virgin copy
                try:
                    session.delete(result)
                except Exception as exc:
                    pytest.fail(f'delete failure: {exc}')

            # insert fake project
            try:
                session.add(project)
            except Exception as exc:
                pytest.fail(f'insert failure: {exc}')

        except Exception as exc:
            pytest.fail(f'failed to create session: {exc}')
        finally:
            if session is not None:
                # 1. delete fake project, then
                session.close()

    def test_create_alma_project(self):
        session = None
        try:
            session = create_session('SDM')
            project = AlmaTestProject()
            result  = None

            try:
                # see if fake project already in DB
                result = session.query(Project) \
                    .filter(Project.project_code ==
                            project.project_code) \
                    .first()
            except Exception as exc:
                pytest.fail(f'query failure: {exc}')

            if result is not None:
                # delete it so we can insert a virgin copy
                try:
                    session.delete(result)
                except Exception as exc:
                    pytest.fail(f'delete failure: {exc}')

            # insert fake project
            try:
                session.add(project)
            except Exception as exc:
                pytest.fail(f'insert failure: {exc}')

        except Exception as exc:
            pytest.fail(f'failed to create session: {exc}')
        finally:
            if session is not None:
                # 1. delete fake project, then
                session.close()

    ### UTILITIES ###

    @staticmethod
    def get_author_pst_ids(project: Project):
        ''' build list of pst_person_ids for display;
            we want the PI's pst_person_id followed by the CoIs' pst_person_ids in numeric order
        '''
        project_pi = project.authors[0]
        coi_pst_ids = [int(coi.pst_person_id) for coi in project.authors[1:]]
        coi_pst_ids = sorted(coi_pst_ids)

        author_pst_ids = [int(project_pi.pst_person_id)]
        [author_pst_ids.append(id) for id in coi_pst_ids]
        return [str(id) for id in author_pst_ids]

    @staticmethod
    def project_is_public(project_code: str):
        ''' is this project accessible by all? '''
        session = create_session('SDM')
        project = session.query(Project) \
            .filter(Project.project_code == project_code) \
            .first()
        if project.proprietary_duration == 0:
            return True
        endtime = project.endtime
        duration = project.proprietary_duration
        now = Time.now().mjd
        release_time = endtime + duration
        elapsed_time = now - release_time
        return elapsed_time > 0

    def template(self):
        ''' just for copying & pasting '''
        session = None
        try:
            session = create_session('SDM')
            try:
                # see if fake project already in DB
                pass
            except Exception as exc:
                pytest.fail(f'query failure: {exc}')

            # if it's there,
            try:
                # delete fake project
                pass
            except Exception as exc:
                pytest.fail(f'delete failure: {exc}')

            try:
                # insert fake project
                pass
            except Exception as exc:
                pytest.fail(f'insert failure: {exc}')

            try:
                # exercise feature
                pass
            except Exception as exc:
                pytest.fail(f'TODO: {exc}')

        except Exception as exc:
            pytest.fail(f'failed to create session: {exc}')
        finally:
            if session is not None:
                # 1. delete fake project, then
                session.close()

    def debug_multiple_fetch_create_session(self):
        ''' for debugging connection issues;
            to use, change "debug" to "test"
        '''
        for _ in range(20):
            session = None
            try:
                try:
                    session = create_session('SDM')
                except Exception as exc:
                    pytest.fail(f'failed to create session: {exc}')

                try:
                    session.query("SELECT starttime FROM projects WHERE "
                                  "project_code = '13B-014'")
                except Exception as exc:
                    pytest.fail(f'query failure: {exc}')
            finally:
                if session is not None:
                    session.close()


if __name__ == '__main__':
    unittest.main()
